@@ -21,16 +21,13 @@ LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 **********************************************************************/
-
 /*****************************************
-*  BDD Package (SAPPORO-1.56)   - Body   *
-*  (C) Shin-ichi MINATO  (Dec. 13, 2012) *
+*  BDD Package (SAPPORO-1.57)   - Body   *
+*  (C) Shin-ichi MINATO  (June 14, 2013) *
 ******************************************/
 
-#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include "SAPPOROBDD/bddc.h"
 
 /* ----------------- MACRO Definitions ---------------- */
@@ -62,11 +59,20 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
   (type *)realloc(ptr, sizeof(type) * size)
 
 /* Printf format of bddp */
-#define B_BDDP_FD "%lld"
-#define B_BDDP_FX "0x%llx"
+#ifdef B_64
+#  define B_BDDP_FD "%lld"
+#  define B_BDDP_FX "0x%llX"
+#else
+#  define B_BDDP_FD "%d"
+#  define B_BDDP_FX "0x%X"
+#endif
 
 /* strtol or strtoll */
-#define B_STRTOI strtoll
+#ifdef B_64
+#  define B_STRTOI strtoll
+#else
+#  define B_STRTOI strtol
+#endif
 
 /* Table spaces */
 #define B_NODE_MAX (B_VAL_MASK>>1U) /* Max number of BDD nodes */
@@ -90,16 +96,23 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define B_BDDP_NP(p)  ((bddp)((p)-Node) << 1U)
 
 /* Read & Write of bddp field in the tables */
-#define B_LOW32(f) ((bddp_32)((f)&((1ULL<<32U)-1U)))
-#define B_HIGH8(f) ((bddp_h8)((f)>>32U))
-#define B_SET_NXP(p, f, i) \
+#ifdef B_64
+#  define B_LOW32(f) ((bddp_32)((f)&((1ULL<<32U)-1U)))
+#  define B_HIGH8(f) ((bddp_h8)((f)>>32U))
+#  define B_SET_NXP(p, f, i) \
     (p ## _h8 = f ## _h8 + i, p ## _32 = f ## _32 + i)
-#define B_GET_BDDP(f) \
+#  define B_GET_BDDP(f) \
     ((bddp) f ## _32 | ((bddp) f ## _h8 << 32U))
-#define B_SET_BDDP(f, g) \
+#  define B_SET_BDDP(f, g) \
     (f ## _h8 = B_HIGH8(g), f ## _32 = B_LOW32(g))
-#define B_CPY_BDDP(f, g) \
+#  define B_CPY_BDDP(f, g) \
     (f ## _h8 = g ## _h8, f ## _32 = g ## _32)
+#else
+#  define B_SET_NXP(p, f, i) (p ## _32 = f ## _32 + i)
+#  define B_GET_BDDP(f) (f ## _32)
+#  define B_SET_BDDP(f, g) (f ## _32 = g)
+#  define B_CPY_BDDP(f, g) (f ## _32 = g ## _32)
+#endif /* B_64 */
 
 /* var & rfc manipulation */
 #define B_VAR_NP(p)    ((p)->varrfc & B_VAR_MASK)
@@ -118,7 +131,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
     ((p)->varrfc -= B_RFC_UNIT, 0))
 
 /* ----------- Stack overflow limitter ------------ */
-const int BDD_RecurLimit = 65536;
+const int BDD_RecurLimit = 8192;
 int BDD_RecurCount = 0;
 #define BDD_RECUR_INC \
   {if(++BDD_RecurCount >= BDD_RecurLimit) \
@@ -147,7 +160,9 @@ int BDD_RecurCount = 0;
 /* ------- Declaration of static (internal) data ------- */
 /* typedef of bddp field in the tables */
 typedef unsigned int bddp_32;
-typedef unsigned char bddp_h8;
+#ifdef B_64
+  typedef unsigned char bddp_h8;
+#endif
 
 /* Declaration of Node table */
 struct B_NodeTable
@@ -156,9 +171,11 @@ struct B_NodeTable
   bddp_32      f1_32;  /* 1-edge */
   bddp_32      nx_32;  /* Node index */
   unsigned int varrfc; /* VarID & Reference counter */
+#ifdef B_64
   bddp_h8      f0_h8;  /* Extention of 0-edge */
   bddp_h8      f1_h8;  /* Extention of 1-edge */
   bddp_h8      nx_h8;  /* Extention of node index */
+#endif /* B_64 */
 };
 static struct B_NodeTable *Node = 0; /* Node Table */
 static bddp NodeLimit = 0;    /* Final limit size */
@@ -173,7 +190,9 @@ struct B_VarTable
   bddp    hashUsed;  /* Current used entries */
   bddvar  lev;      /* Level of the variable */
   bddp_32 *hash_32; /* Hash-table */
+#ifdef B_64
   bddp_h8 *hash_h8; /* Extension of hash-table */
+#endif /* B_64 */
 };
 static struct B_VarTable *Var = 0; /* Var-tables */
 static bddvar *VarID = 0;     /* VarID reverse table */
@@ -187,9 +206,11 @@ struct B_CacheTable
   bddp_32       g_32; /* an operand BDD */
   bddp_32       h_32; /* Result BDD */
   unsigned char op;   /* Operation code */
+#ifdef B_64
   bddp_h8       f_h8; /* Extention of an operand BDD */
   bddp_h8       g_h8; /* Extention of an operand BDD */
   bddp_h8       h_h8; /* Extention of result BDD */
+#endif /* B_64 */
 };
 static struct B_CacheTable *Cache = 0; /* Opeartion cache */
 static bddp CacheSpc = 0;           /* Current cache size */
@@ -199,8 +220,10 @@ struct B_RFC_Table
 {
   bddp_32 nx_32;   /* Node index */
   bddp_32 rfc_32;  /* RFC */
+#ifdef B_64
   bddp_h8 nx_h8;   /* Extension of Node index */
   bddp_h8 rfc_h8;  /* Extension of RFC */
+#endif /* B_64 */
 };
 static struct B_RFC_Table *RFCT = 0; /* RFC-Table */
 static bddp RFCT_Spc;   /* Current RFC-table size */
@@ -224,7 +247,7 @@ static void dump B_ARG((bddp f));
 static void reset B_ARG((bddp f));
 static void export B_ARG((FILE *strm, bddp f));
 static int import B_ARG((FILE *strm, bddp *p, int lim, int z));
-static bddp andfalse B_ARG((bddp f, bddp g));
+static int andfalse B_ARG((bddp f, bddp g));
 
 
 /* ------------------ Body of program -------------------- */
@@ -244,7 +267,9 @@ bddp limitsize;
     for(i=0; i<VarSpc; i++)
     {
       if(Var[i].hash_32) free(Var[i].hash_32);
+#ifdef B_64
       if(Var[i].hash_h8) free(Var[i].hash_h8);
+#endif
     }
     free(Var);
   }
@@ -305,7 +330,9 @@ bddp limitsize;
     Var[i].lev = i;
     VarID[i] = i;
     Var[i].hash_32 = 0;
+#ifdef B_64
     Var[i].hash_h8 = 0;
+#endif
   }
 
   for(ix=0; ix<CacheSpc; ix++) Cache[ix].op = BC_NULL;
@@ -352,9 +379,11 @@ int bddgc()
   struct B_NodeTable *np;
   struct B_VarTable *varp;
   bddvar v;
-  bddp oldSpc, newSpc, nx, key/*, f0, f1*/;
+  bddp oldSpc, newSpc, nx, key, f0, f1;
   bddp_32 *newhash_32, *p_32, *p2_32;
+#ifdef B_64
   bddp_h8 *newhash_h8, *p_h8, *p2_h8;
+#endif
 
   n = NodeUsed; 
   for(fp=Node; fp<Node+NodeSpc; fp++)
@@ -443,6 +472,7 @@ int bddgc()
     if(newSpc == oldSpc) continue;
 
     /* Reduce space */
+#ifdef B_64
     newhash_32 = B_MALLOC(bddp_32, newSpc);
     newhash_h8 = B_MALLOC(bddp_h8, newSpc);
     if(!newhash_32 || !newhash_h8)
@@ -451,6 +481,10 @@ int bddgc()
       if(newhash_h8) free(newhash_h8);
       break; /* Not enough memory */
     }
+#else
+    newhash_32 = B_MALLOC(bddp_32, newSpc);
+    if(!newhash_32) break; /* Not enough memory */
+#endif
 
     /* Initialize new hash entry */
     for(i=0; i<newSpc; i++)
@@ -482,8 +516,10 @@ int bddgc()
     varp->hashSpc = newSpc;
     free(varp->hash_32);
     varp->hash_32 = newhash_32;
+#ifdef B_64
     free(varp->hash_h8);
     varp->hash_h8 = newhash_h8;
+#endif
   }
   return 0;
 }
@@ -703,7 +739,7 @@ bddvar bddnewvar()
 bddvar bddnewvaroflev(lev)
 bddvar  lev;
 {
-  bddvar i/*, v*/;
+  bddvar i, v;
 
   if(lev == 0 || lev > ++VarUsed)
     err("bddnewvaroflev: Invalid level", lev);
@@ -914,14 +950,14 @@ bddp f, g;
   return B_NOT(h);
 }
 
-bddp bddimply(f, g)
+int bddimply(f, g)
 bddp f, g;
 {
   struct B_NodeTable *fp;
    
   /* Check operands */
-  if(f == bddnull) return bddnull;
-  if(g == bddnull) return bddnull;
+  if(f == bddnull) return 0;
+  if(g == bddnull) return 0;
   if(B_CST(f))
   { if(B_ABS(f) != bddfalse) err("bddimply: Invalid bddp", f); }
   else
@@ -941,7 +977,7 @@ bddp f, g;
     if(B_Z_NP(fp)) err("bddimply: applying ZBDD node", g);
   }
 
-  return B_NOT(andfalse(f, B_NOT(g)));
+  return ! andfalse(f, B_NOT(g));
 }
 
 bddp bddsupport(f)
@@ -1002,7 +1038,7 @@ bddvar shift;
 /* Returns bddnull if not enough memory */
 {
   struct B_NodeTable *fp;
-  /*bddvar flev;*/
+  bddvar flev;
 
   /* Check operands */
   if(shift >= VarUsed)
@@ -1022,7 +1058,7 @@ bddvar shift;
 /* Returns bddnull if not enough memory */
 {
   struct B_NodeTable *fp;
-  /*bddvar flev;*/
+  bddvar flev;
 
   /* Check operands */
   if(shift >= VarUsed)
@@ -1253,7 +1289,7 @@ static void var_enlarge()
 {
   bddvar i, newSpc;
   struct B_VarTable *newVar;
-  bddvar *newVarID;
+  unsigned int *newVarID;
   
   /* Get new size */
   if(VarSpc == bddvarmax+1U)
@@ -1263,7 +1299,7 @@ static void var_enlarge()
 
   /* Enlarge space */
   newVar = B_MALLOC(struct B_VarTable, newSpc);
-  newVarID = B_MALLOC(bddvar, newSpc);
+  newVarID = B_MALLOC(unsigned int, newSpc);
   if(newVar && newVarID)
   {
     for(i=0; i<VarSpc; i++)
@@ -1273,7 +1309,9 @@ static void var_enlarge()
       newVar[i].lev = Var[i].lev;
       newVar[i].hash_32 = Var[i].hash_32;
       newVarID[i] = VarID[i];
+#ifdef B_64
       newVar[i].hash_h8 = Var[i].hash_h8;
+#endif
     }
     free(Var);
     free(VarID);
@@ -1295,7 +1333,9 @@ static void var_enlarge()
     Var[i].lev = i;
     Var[i].hash_32 = 0;
     VarID[i] = i;
+#ifdef B_64
     Var[i].hash_h8 = 0;
+#endif
   }
   VarSpc = newSpc;
 }
@@ -1322,9 +1362,11 @@ static int node_enlarge()
       newNode[i].f0_32 = Node[i].f0_32;
       newNode[i].f1_32 = Node[i].f1_32;
       newNode[i].nx_32 = Node[i].nx_32;
+#ifdef B_64
       newNode[i].f0_h8 = Node[i].f0_h8;
       newNode[i].f1_h8 = Node[i].f1_h8;
       newNode[i].nx_h8 = Node[i].nx_h8;
+#endif /* B_64 */
     }
     free(Node);
     Node = newNode;
@@ -1385,7 +1427,9 @@ bddvar v;
   struct B_VarTable *varp;
   bddp i, oldSpc, newSpc, nx, key, f0, f1;
   bddp_32 *newhash_32, *p_32;
+#ifdef B_64
   bddp_h8 *newhash_h8, *p_h8;
+#endif
   
   varp = &Var[v];
   /* Get new size */
@@ -1395,6 +1439,7 @@ bddvar v;
   newSpc = oldSpc << 1U;
 
   /* Enlarge space */
+#ifdef B_64
   newhash_32 = B_MALLOC(bddp_32, newSpc);
   newhash_h8 = B_MALLOC(bddp_h8, newSpc);
   if(newhash_32 && newhash_h8)
@@ -1415,6 +1460,16 @@ bddvar v;
     if(newhash_h8) free(newhash_h8);
     return 1;
   }
+#else
+  newhash_32 = B_MALLOC(bddp_32, newSpc);
+  if(newhash_32)
+  {
+    for(i=0; i<varp->hashSpc; i++) newhash_32[i] = varp->hash_32[i];
+    free(varp->hash_32);
+    varp->hash_32 = newhash_32;
+  }
+  else return 1; /* Not enough memory */
+#endif
   varp->hashSpc = newSpc;
 
   /* Initialize new hash entry */
@@ -1462,7 +1517,9 @@ bddp    f0, f1;
   struct B_VarTable *varp;
   bddp ix, nx, key;
   bddp_32 *p_32;
+#ifdef B_64
   bddp_h8 *p_h8;
+#endif
 
   varp = &Var[v];
   if(varp->hashSpc == 0)
@@ -1470,12 +1527,14 @@ bddp    f0, f1;
   {
     varp->hash_32 = B_MALLOC(bddp_32, B_HASH_SPC0);
     if(!varp->hash_32) return bddnull;
+#ifdef B_64
     varp->hash_h8 = B_MALLOC(bddp_h8, B_HASH_SPC0);
     if(!varp->hash_h8)
     {
       free(varp->hash_32);
       return bddnull;
     }
+#endif
     for(ix=0; ix<B_HASH_SPC0; ix++)
     {
       B_SET_NXP(p, varp->hash, ix);
@@ -1572,9 +1631,9 @@ unsigned char op, skip;
 {
   struct B_NodeTable *fp, *gp;
   struct B_CacheTable *cachep;
-  bddp key = 0, f0 = 0, f1 = 0, g0 = 0, g1 = 0, h0, h1, h = 0;
-  bddvar v = 0, flev, glev;
-  char z = 0; /* flag to check ZBDD node */
+  bddp key, f0, f1, g0, g1, h0, h1, h;
+  bddvar v, flev, glev;
+  char z; /* flag to check ZBDD node */
 
   /* Check terminal case */
   if(!skip) switch(op)
@@ -2123,7 +2182,9 @@ struct B_NodeTable *np;
   struct B_VarTable *varp;
   struct B_NodeTable *np1, *np2;
   bddp_32 *p_32;
+#ifdef B_64
   bddp_h8 *p_h8;
+#endif
 
   /* remove the node from hash list */
   varp = Var + B_VAR_NP(np);
@@ -2177,10 +2238,10 @@ static bddp count(f)
 bddp f;
 {
   bddp nx;
-  bddp c/*, g*/;
-  /*bddvar flev, glev;*/
+  bddp c, g;
+  bddvar flev, glev;
   struct B_NodeTable *fp;
-  /*struct B_NodeTable *gp;*/
+  struct B_NodeTable *gp;
 
   /* Check consistensy
   if(f == bddnull)
@@ -2332,7 +2393,7 @@ bddvar v;
 bddp f0, f1;
 /* Returns bddnull if not enough memory */
 {
-    /*struct B_NodeTable *fp;*/
+  struct B_NodeTable *fp;
 
   /* Check elimination rule */
   if(f1 == bddfalse) return f0;
@@ -2349,20 +2410,17 @@ bddp f0, f1;
   return getnode(v, B_NOT(f0), f1);
 }
 
-static bddp andfalse(f, g)
+static int andfalse(f, g)
 bddp f, g;
-/* Returns bddnull if not enough memory */
 {
   struct B_NodeTable *fp, *gp;
   struct B_CacheTable *cachep;
-  bddp key, f0, f1, g0, g1/*, h0, h1*/, h;
-  bddvar /*v,*/ flev, glev;
+  bddp key, f0, f1, g0, g1, h0, h1, h;
+  bddvar v, flev, glev;
 
   /* Check trivial cases */
-  if(f == bddfalse || g == bddfalse || f == B_NOT(g))
-    return bddfalse;
-  if(f == bddtrue || g == bddtrue || f == g)
-    return bddtrue;
+  if(f == bddfalse || g == bddfalse || f == B_NOT(g)) return 0;
+  if(f == bddtrue || g == bddtrue || f == g) return 1;
   /* Check operand swap */
   if(f > g) { h = f; f = g; g = h; } /* swap (f, g) */
 
@@ -2381,7 +2439,7 @@ bddp f, g;
     {
       /* Hit */
       h = B_GET_BDDP(cachep->h);
-      return (h==bddfalse)? bddfalse: bddtrue;
+      return (h==bddfalse)? 0: 1;
     }
   }
   /* Get (f0, f1) and (g0, g1)*/
@@ -2394,7 +2452,7 @@ bddp f, g;
 
   if(flev <= glev)
   {
-    /*v = B_VAR_NP(gp);*/
+    v = B_VAR_NP(gp);
     g0 = B_GET_BDDP(gp->f0);
     g1 = B_GET_BDDP(gp->f1);
     if(B_NEG(g)) { g0 = B_NOT(g0); g1 = B_NOT(g1); }
@@ -2402,15 +2460,15 @@ bddp f, g;
 
   if(flev >= glev)
   {
-    /*v = B_VAR_NP(fp);*/
+    v = B_VAR_NP(fp);
     f0 = B_GET_BDDP(fp->f0);
     f1 = B_GET_BDDP(fp->f1);
     if(B_NEG(f)) { f0 = B_NOT(f0); f1 = B_NOT(f1); }
   }
 
   /* Get result */
-  if(andfalse(f0, g0) == bddtrue) return bddtrue;
-  if(andfalse(f1, g1) == bddtrue) return bddtrue;
+  if(andfalse(f0, g0) == 1) return 1;
+  if(andfalse(f1, g1) == 1) return 1;
 
   /* Saving to Cache */
   if(key != bddnull)
@@ -2421,7 +2479,7 @@ bddp f, g;
     B_SET_BDDP(cachep->g, g);
     B_SET_BDDP(cachep->h, bddfalse);
   }
-  return bddfalse;
+  return 0;
 }
 
 static int err(msg, num)
@@ -2449,7 +2507,7 @@ static int rfc_inc_ovf(np)
 struct B_NodeTable *np;
 {
   bddp ix, nx, nx2, key, rfc, oldSpc;
-  /*bddp *p, *p2;*/
+  bddp *p, *p2;
   struct B_RFC_Table *oldRFCT;
 
 /* printf("rfc_inc %d (u:%d)\n", np-Node, RFCT_Used); */
